package com.ibm.mobileservices.sample.VNurseCL;
import com.ibm.db2ess.cl.*;

/* IBM Confidential
 *
 * Product ID: 5724-D04
 *
 * (C) Copyright IBM Corp. 2003
 *
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */

/**
 * <pre>
 * Descriptive Name:  Custom Logic during the replication of one specific table.
 *
 * Description:
 *    This class is called by Sync Server during the replication of one specific table.
 *		For demonstration purpose, this Custom Logic selects a winner from the device users
 *    and overrid the default rule (source as winner).
 *
 *    Modifications:
 *
 *    Date      Author                 	Category             Defect/Feature
 *              Description
 *
 *  2003/04/01  Thanh Pham             	New Code             CustomLogic
 *              Initial code.
 *
 * </pre>
 *
 *    @see com.ibm.db2ess.cl.TableReplicatePolicy
 * 		@see com.ibm.db2ess.cl.CLException
 *    @author Thanh Pham
 */
public class VNurseTableReplicatePolicy extends TablePolicy {

  /**
   * This method will be invoked by the SyncServer during the replication of one specific table.
   *
   * @exception com.ibm.db2ess.cl.CLException
   * Custom logic generated or encountered exception.
   */
  public void execute(CLRecord iRecord) throws CLException {

		SampleDataLog sdl = SampleDataLog.getInstance();
		String s = new String("\n=====> TableReplicatePolicy");

		sdl.writeOut(s);

		if (iRecord != null) {
			// Log the data
			sdl.log(iRecord, sdl.LOG_ALL);

			// Detect the conflict and manage it
			if (iRecord.conflictExists()) {
				sdl.writeOut("=================== Conflit exists ==================");

				// Go thru the candidate to select the winner
				// Get the candidate array and process it
				CLCandidate[] clCandidateArray = iRecord.getCandidates();
				if (clCandidateArray != null) {
					// Scan all the the CLCandidates
					for (int i=0; i < clCandidateArray.length; i++) {
						// Change the default conflict resolution rule by selecting the winner from users instead of source
						// When there are several candidate from users, just pick one

						if (clCandidateArray[i].fromUser()) {
							sdl.writeOut("Found a Custom Logic winner candidate");
							sdl.log(clCandidateArray[i], sdl.LOG_ALL);

							// Change the prevailing candidate to this one
							iRecord.force(clCandidateArray[i]);
							break;
						}
					}
				} else {
					sdl.writeOut("!!! ERROR !!! - Conflicts with no candidate ");
				}
			}

			// Apply the Custom Logic on this record
			try {
				iRecord.apply();
				if (iRecord.applied()) {
					sdl.writeOut("Record applied successfully");
				} else {
					sdl.writeOut("Record applied fail");
				}
			}
			catch (CLCandidateException clCandidateException) {
				sdl.log(clCandidateException, sdl.LOG_ALL);
			}
		} else {
			sdl.writeOut("No CLRecord input parameter.");
		}

	}

} 